package function

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"

	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/transactionCC/mngr"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
)

func Transfer(userParameter *requests.Parameter, systemParameter *requests.Parameter, manager chaincode.IManager) (interface{}, error) {
	var txn transactionCC.TxnFundsLockInput
	err := json.Unmarshal(userParameter.Param, &txn)
	if err != nil {
		return nil, errors.Errorf("Can't unmarshal userParam to TxnFundsLockInput. Error: %s", err)
	}

	if txn.InUser == nil {
		return nil, errors.Errorf("In user can't by null or empty")
	}

	if txn.OutUser == nil {
		return nil, errors.Errorf("Out user can't by null")
	}

	if len(txn.OutUser.UserID) == 0 {
		return nil, errors.Errorf("Out user can't by empty")
	}

	if txn.OutUser.UserID != userParameter.UserID {
		return nil, errors.Errorf("Can't %s because signer UserID %s not equals with locked founds UserID %s", invoke.LockFundsOnAccountFunction, userParameter.UserID, txn.OutUser.UserID)
	}

	if txn.OutUser.UserID == txn.InUser.UserID &&
		txn.ProcessType == transactionCC.ExchangeWithBothCommission_ProcessType {
		return nil, errors.Errorf("%s. Transaction with processType 'ExchangeWithBothCommission' out user id {%s} and in user id {%s} can't be equals %s", invoke.LockFundsOnAccountFunction, txn.OutUser.UserID, txn.InUser.UserID)
	}

	var lockFundsOnAccountArg transactionCC.LockFundsOnAccountArg
	err = json.Unmarshal(systemParameter.Param, &lockFundsOnAccountArg)
	if err != nil {
		return nil, errors.Errorf("Can't unmarshal systemParams to LockFundsOnAccountArg. Error %s", err)
	}

	txn.OperationID = lockFundsOnAccountArg.OperationID

	transactionList, err := LockFundsOnAccount(userParameter, systemParameter, manager)
	if err != nil {
		return transactionList, err
	}

	transfer := &transactionCC.Transfer{}
	transfer.OperationID = txn.OperationID
	transfer.UserAccountInfo = *txn.OutUser
	return mngr.GetTransactionManager(manager).TransferFunds(transfer)
}
