package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
)

func (this *TransactionManagerImpl) accountCurrencyBalance(userAccountInfo *accountCC.UserAccountInfo, CurrencyType string) (*transactionCC.AccountBalance, error) {
	if userAccountInfo == nil {
		return nil, logging.Get().Error("UserAccountInfo can't be nil")
	}

	err := userAccountInfo.Validate()
	if err != nil {
		return nil, logging.Get().Errorf("userAccountInfo is not valid, %s", err.Error())
	}

	accountCurrencyBalanceArg := transactionCC.AccountCurrencyBalanceArg{}

	accountCurrencyBalanceArg.UserAccountInfo = *userAccountInfo
	accountCurrencyBalanceArg.CurrencyType = CurrencyType
	return this.AccountCurrencyBalance(accountCurrencyBalanceArg)
}

// AccountCurrencyBalance compute user balance for given currency
func (this *TransactionManagerImpl) AccountCurrencyBalance(accountCurrencyBalanceArg transactionCC.AccountCurrencyBalanceArg) (*transactionCC.AccountBalance, error) {
	currencyType := accountCurrencyBalanceArg.CurrencyType

	chaincode.GlobalLogger.Infof("AccountCurrencyBalance. userID %s, walletID %s, accountID %s, currencyType %s",
		accountCurrencyBalanceArg.UserAccountInfo.UserID, accountCurrencyBalanceArg.UserAccountInfo.WalletID, accountCurrencyBalanceArg.UserAccountInfo.AccountID, currencyType)
	var total, err = nums.NumFromInt(0, currencyType)
	if err != nil {
		return nil, err
	}

	inVoList, err := this.GetObjectListByKeys(&transactionCC.TransactionRecord{}, []string{accountCurrencyBalanceArg.UserAccountInfo.UserID, accountCurrencyBalanceArg.UserAccountInfo.WalletID, accountCurrencyBalanceArg.UserAccountInfo.AccountID, currencyType, transactionCC.INPUT.String()})
	if err != nil {
		return nil, err
	}

	chaincode.GlobalLogger.Info(transactionCC.INPUT.String())
	for _, inVo := range inVoList {
		transactionRecord, err := transactionCC.Cast(inVo)
		if err != nil {
			return nil, errors.Errorf("failed to unmarshal transaction record, for userID %s, walletID %s, accountID %s, currencyType %s, due to %s", accountCurrencyBalanceArg.UserAccountInfo.UserID, accountCurrencyBalanceArg.UserAccountInfo.WalletID, accountCurrencyBalanceArg.UserAccountInfo.AccountID, err)
		}

		number, err := transactionRecord.Amount.Number()
		if err != nil {
			return nil, err
		}

		chaincode.GlobalLogger.Infof("+ %s - transaction key %s", number.Num.String(), transactionRecord.Key)
		_, err = total.AddSelf(number)
		if err != nil {
			return nil, err
		}

	}

	chaincode.GlobalLogger.Info(transactionCC.OUTPUT.String())
	outVoList, err := this.GetObjectListByKeys(&transactionCC.TransactionRecord{}, []string{accountCurrencyBalanceArg.UserAccountInfo.UserID, accountCurrencyBalanceArg.UserAccountInfo.WalletID, accountCurrencyBalanceArg.UserAccountInfo.AccountID, currencyType, transactionCC.OUTPUT.String()})
	if err != nil {
		return nil, err
	}

	for _, outVo := range outVoList {
		transactionRecord, err := transactionCC.Cast(outVo)
		if err != nil {
			return nil, errors.Errorf("failed to unmarshal transaction record, for userID %s, walletID %s, accountID %s, currencyType %s, due to %s", accountCurrencyBalanceArg.UserAccountInfo.UserID, accountCurrencyBalanceArg.UserAccountInfo.WalletID, accountCurrencyBalanceArg.UserAccountInfo.AccountID, err)
		}

		number, err := transactionRecord.Amount.Number()
		if err != nil {
			return nil, err
		}

		chaincode.GlobalLogger.Infof("- %s - transaction key %s", number.Num.String(), transactionRecord.Key)
		_, err = total.SubSelf(number)
		if err != nil {
			return nil, err
		}

	}

	accountBalance := &transactionCC.AccountBalance{
		UserAccountInfo: accountCurrencyBalanceArg.UserAccountInfo,
		Balance:         total.Plain(),
	}
	return accountBalance, nil
}
