package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
)

// CheckExistsTransactionByList Проверка списка на транзакций на существования в канале transactionchannel по operationID
func (this *TransactionManagerImpl) CheckExistsTransactionByList(syncRequest *transactionCC.SyncRequest) (*transactionCC.CheckExistsResult, error) {
	chaincode.GlobalLogger.Infof("CheckExistsTransactionByList")

	externalTransactionInfoList := syncRequest.ExternalTransactionInfoList
	checkExistsResult := transactionCC.CheckExistsResult{}

	// если передан пустой список тогда возращаем пустой результат
	lenList := len(externalTransactionInfoList)
	if lenList == 0 {
		chaincode.GlobalLogger.Info("ExternalTransactionInfoList is empty.")
		return &checkExistsResult, nil
	}

	checkExistsResult.CheckExistsExternalTransactionResultList = make([]transactionCC.CheckExistsExternalTransactionResult, lenList)
	for _, externalTransactionInfo := range externalTransactionInfoList {
		chaincode.GlobalLogger.Infof("externalTransactionInfo: %s", externalTransactionInfo.ExternalTransactionID)
		chaincode.GlobalLogger.Infof("check by OperationID: %s", externalTransactionInfo.OperationID)

		item := transactionCC.CheckExistsExternalTransactionResult{}
		item.ExternalTransactionID = externalTransactionInfo.ExternalTransactionID

		// проверка существования транзакции по operationID
		isExistsTransactionByOperationId, err := isExistsTransactionByOperationId(externalTransactionInfo.OperationID, this)
		if err != nil {
			item.Status = transactionCC.ERROR
			item.Message = err.Error()
		}

		if !isExistsTransactionByOperationId {
			item.Status = transactionCC.NOT_FOUND
		} else {
			item.Status = transactionCC.ALREADY_EXISTS
		}

		chaincode.GlobalLogger.Infof("Result status: %s", item.Status)

		checkExistsResult.CheckExistsExternalTransactionResultList = append(checkExistsResult.CheckExistsExternalTransactionResultList, item)
	}

	return &checkExistsResult, nil
}
