package mngr

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_cryptocurrency"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

//
// метод для ввода средств в crypotenter (в transactionchannel) через выписки со счета в других валютах
//
// systemUserID - идентификатор демона для ввода средств
// channelName - название канала в котором проверять средства для ввода
// chaincodeName - название смартконтракта который вызывать для ввода средств
//
// пример: для ввода Bitcoin
//
//	BitcoinCC      = "bitcoinCC"
//	BitcoinChannel      = "bitcoinchannel"
//
// пример: для ввода Ethereum
//
//	EthereumCC      = "ethereumCC"
//	EthereumChannel      = "ethereumchannel"
//
// TODO добавить проверку допустимых смартконтрактов передаваемых в этот метод
//
func (this *TransactionManagerImpl) DepositFunds(systemUserID string, invokeChaincode *transactionCC.InvokeChaincode) (*transactionCC.LoadCryptocurrencyResultList, error) {
	chaincode.GlobalLogger.Info("transactionCC.depositFunds START")

	// загрузка новых криптовалют из канала cryptocurrencychannel
	// todo изменить проброс через systemParameter.UserID вместо Param
	remoteManager := remote_cryptocurrency.GetCryptocurrencyRemoteManager(this, invokeChaincode.Channel, invokeChaincode.Chaincode)
	cryptocurrencyStoreOutput, err := remoteManager.GetCryptocurrenciesByDaemonName(&requests.Id{Id: systemUserID})
	if err != nil {
		return nil, errors.Errorf("failed to load cryptocurrency due to, %s", err)
	}

	loadCryptocurrencyResultList := transactionCC.LoadCryptocurrencyResultList{
		LoadCryptocurrencyResultList: []transactionCC.LoadCryptocurrencyResult{},
	}
	for _, cryptocurrencyStore := range cryptocurrencyStoreOutput.CryptocurrencyStoreList {
		var transactionRecordList []*transactionCC.TransactionRecord
		if invokeChaincode.Chaincode == invoke.BitcoinCC  {
			transactionRecordList, err = this.depositBitcoinFundsTransaction(cryptocurrencyStore)
		} else if invokeChaincode.Chaincode == invoke.EthereumCC {
			transactionRecordList, err = this.depositEthereumFundsTransaction(cryptocurrencyStore)
		} else {
			return nil, errors.Errorf("unknown currency type %s", err)
		}

		if err != nil {
			// Update results with an error record
			loadCryptocurrencyResultList.LoadCryptocurrencyResultList = append(loadCryptocurrencyResultList.LoadCryptocurrencyResultList, transactionCC.LoadCryptocurrencyResult{
				ExternalTransactionId: cryptocurrencyStore.ExternalTransactionID,
				TransactionRecordList: transactionRecordList,
				Status:                transactionCC.Error,
				CommonMessage:         "Can't save transaction",
				DetailMessage:         err.Error(),
			})
		} else {
			loadCryptocurrencyResultList.LoadCryptocurrencyResultList = append(loadCryptocurrencyResultList.LoadCryptocurrencyResultList, transactionCC.LoadCryptocurrencyResult{
				ExternalTransactionId: cryptocurrencyStore.ExternalTransactionID,
				TransactionRecordList: transactionRecordList,
				Status:                transactionCC.Ok,
				CommonMessage:         "",
				DetailMessage:         "",
			})
		}
	}

	return &loadCryptocurrencyResultList, nil
}
