package mngr

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
)

func (this *TransactionManagerImpl) FindLockAmountByOperationIdAndUserIdAndCurrencyType(findLockAmountByOperationIDAndUserIdArg transactionCC.FindLockAmountByOperationIdAndUserIdAndCurrencyTypeArg) (*transactionCC.FindLockAmountResult, error) {
	if len(findLockAmountByOperationIDAndUserIdArg.Amount.CurrencyType) == 0 || len(findLockAmountByOperationIDAndUserIdArg.Amount.Amount) == 0 {
		return nil, errors.Errorf("amount can't be nil")
	}

	chaincode.GlobalLogger.Infof("FindLockAmountByOperationIdAndUserIdAndCurrencyType(operationID %s, UserAccountInfo %s)",
		findLockAmountByOperationIDAndUserIdArg.OperationID, findLockAmountByOperationIDAndUserIdArg.UserAccountInfo.String())

	err := findLockAmountByOperationIDAndUserIdArg.UserAccountInfo.Validate()
	if err != nil {
		return nil, err
	}

	if len(findLockAmountByOperationIDAndUserIdArg.OperationID) == 0 {
		errMsg := "operationID is empty"

		return nil, errors.Errorf(errMsg)
	}

	totalLockAmount, err := nums.NumFromInt(0, findLockAmountByOperationIDAndUserIdArg.Amount.CurrencyType)
	if err != nil {
		return nil, err
	}

	outValidObjects, err := this.GetObjectListByKeysWithFilter(&transactionCC.TransactionRecord{},
			[]string{findLockAmountByOperationIDAndUserIdArg.UserAccountInfo.UserID, findLockAmountByOperationIDAndUserIdArg.UserAccountInfo.WalletID, findLockAmountByOperationIDAndUserIdArg.UserAccountInfo.AccountID, findLockAmountByOperationIDAndUserIdArg.Amount.CurrencyType, transactionCC.OUTPUT.String(), findLockAmountByOperationIDAndUserIdArg.OperationID, transactionCC.LockAmount.String()})
	if err != nil {
		return nil, err
	}

	for _, vo := range outValidObjects {
		transactionRecord, err := transactionCC.Cast(vo)
		if err != nil {
			return nil, err
		}

		chaincode.GlobalLogger.Infof("found transaction %s", transactionRecord.Key)
		chaincode.GlobalLogger.Infof("Amount %s", transactionRecord.Amount)
		currencyNumber, err := transactionRecord.Amount.Number()
		if err != nil {
			return nil, err
		}
		chaincode.GlobalLogger.Infof("Amount.Number %s", currencyNumber.RoundBankString())

		number, err := transactionRecord.Amount.Number()
		if err != nil {
			return nil, err
		}
		_, err = totalLockAmount.AddSelf(number)
		if err != nil {
			return nil, err
		}
		chaincode.GlobalLogger.Infof("acc %s", totalLockAmount)
		chaincode.GlobalLogger.Infof("acc.Plain() %s", totalLockAmount.Plain())
	}

	lockAmountResult := transactionCC.FindLockAmountResult{}
	lockAmountResult.Amount = totalLockAmount.Plain()
	chaincode.GlobalLogger.Infof("lockAmountResult.Amount %s", lockAmountResult.Amount)
	return &lockAmountResult, nil
}
