package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"github.com/pkg/errors"
)

func (this *TransactionManagerImpl) findTransactionListByOperationID(transfer *transactionCC.Transfer) (*transactionCC.UnlockTransactionInfoList, error) {
	chaincode.GlobalLogger.Infof("findTransactionListByOperationID(operationID: %s)", transfer)

	if transfer == nil {
		return nil, chaincode.GlobalLogger.Error("transfer is empty")
	}

	if len(transfer.OperationID) == 0 {
		return nil, chaincode.GlobalLogger.Error("operationID is empty")
	}

	validObjectsFirst, err := this.GetObjectListByKeysWithFilter(&transactionCC.TransactionRecord{}, []string{transfer.UserAccountInfo.UserID, transfer.UserAccountInfo.WalletID, transfer.UserAccountInfo.AccountID}, transfer.OperationID, transactionCC.OUTPUT.String(), transactionCC.LockAmount.String())
	if err != nil {
		return nil, err
	}

	if len(validObjectsFirst) == 0 {
		return nil, errors.Errorf("Can not find %s transaction by operationID: %s, operationType: %s for user info %s", transactionCC.OUTPUT.String(), transfer.OperationID, transactionCC.LockAmount.String(), transfer.UserAccountInfo.String())
	}

	outTransactionRecord, err := transactionCC.Cast(validObjectsFirst[0])
	if err != nil {
		return nil, err
	}

	inTransactionRecord, err := this.FindTransactionRecordByDependencyKey(outTransactionRecord.DependencyKey)
	if err != nil {
		return nil, err
	}

	validObjectsSecond, err := this.GetObjectListByKeysWithFilter(&transactionCC.TransactionRecord{}, []string{inTransactionRecord.DestinationUserAccountInfo.UserID, inTransactionRecord.DestinationUserAccountInfo.WalletID, inTransactionRecord.DestinationUserAccountInfo.AccountID}, transfer.OperationID, transactionCC.OUTPUT.String(), transactionCC.LockAmount.String())
	if err != nil {
		return nil, err
	}

	validObjects := make([]chaincode.ValidObject, 0)
	validObjects = append(validObjects, validObjectsFirst...)
	if len(validObjectsFirst) > 0 {
		validObjects = append(validObjects, validObjectsSecond...)
	}

	var unlockTransactionInfoList transactionCC.UnlockTransactionInfoList
	for _, vo := range validObjects {
		transactionRecord, err := transactionCC.Cast(vo)
		if err != nil {
			return nil, err
		}

		chaincode.GlobalLogger.Infof("found transaction with key %s", transactionRecord.Key)

		lockTransactionRecordInfo := transactionCC.LockTransactionRecordInfo{}
		lockTransactionRecordInfo.OutLockAmountTransactionRecord = transactionRecord

		chaincode.GlobalLogger.Infof("Lock amount transaction record: %s", transactionRecord.Key)

		dependencyTransactionRecord, err := this.FindTransactionRecordByDependencyKey(transactionRecord.DependencyKey)
		if err != nil {
			return nil, err
		}
		lockTransactionRecordInfo.InLockAmountTransactionRecord = dependencyTransactionRecord

		chaincode.GlobalLogger.Infof("Unlock for UserID: %s, AccountID: %s, WalletID: %s", dependencyTransactionRecord.UserAccountInfo.UserID, dependencyTransactionRecord.UserAccountInfo.AccountID, dependencyTransactionRecord.UserAccountInfo.WalletID)

		// для процесса комиссия с отправителя ЕСТЬ транзакции о блокировке комиссии
		// для процесса комиссия с получателя НЕТ транзакции о блокировке комиссии
		// для процесса переводы между своими счетами НЕТ транзакции о блокировке комиссии
		if dependencyTransactionRecord.ProcessType != transactionCC.TransferReceiverCommission_ProcessType && dependencyTransactionRecord.ProcessType != transactionCC.Move_ProcessType {
			outLockedCommission, inLockedCommission, err := this.FindTransactionByOperationIDAndOperationTypeAndUserAccountInfo(transfer.OperationID, transactionCC.LockCommission, &transfer.UserAccountInfo)
			if err != nil {
				return nil, err
			}

			// TODO lockedCommission.Transactions[0] think how to fix
			lockTransactionRecordInfo.InLockCommissionTransactionRecord = inLockedCommission.Transactions[0]
			lockTransactionRecordInfo.OutLockCommissionTransactionRecord = outLockedCommission.Transactions[0]

			chaincode.GlobalLogger.Infof("Found lock commission transaction record with key %s", lockTransactionRecordInfo.InLockCommissionTransactionRecord.Key)
		}

		// добавляем в список транзакцию с комиссией и транзакциями на перевод средств
		unlockTransactionInfoList.LockTransactionRecordInfoList = append(unlockTransactionInfoList.LockTransactionRecordInfoList, lockTransactionRecordInfo)
	}

	chaincode.GlobalLogger.Infof("found LockTransactionRecordInfoList: %s", len(unlockTransactionInfoList.LockTransactionRecordInfoList))

	return &unlockTransactionInfoList, nil
}
