package mngr

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
)

// GetUserTransactions retrieves a list of all users transactions for given user id
func (this *TransactionManagerImpl) GetTransactions(id *requests.Id) (*transactionCC.TransactionList, error) {
	userID := id.Id
	chaincode.GlobalLogger.Infof("GetUserTransactions by userID %s", userID)
	// TODO: need to take care of pagination to retrieve transactions in batches rather all at once
	if len(userID) == 0 {
		errMsg := "an empty user id, cannot process, get transactions request"

		return nil, errors.Errorf(errMsg)
	}

	validObjects, err := this.GetObjectListByKeys(&transactionCC.TransactionRecord{}, []string{userID})
	if err != nil {
		return nil, err
	}

	txs := &transactionCC.TransactionList{}
	txs.Transactions = make([]*transactionCC.TransactionRecord, 0)
	for _, vo := range validObjects {
		transactionRecord, err := transactionCC.Cast(vo)
		if err != nil {
			return nil, err
		}
		// add user transaction into list
		txs.Transactions = append(txs.Transactions, transactionRecord)

		transactionRecord, err = this.FindTransactionRecordByDependencyKey(transactionRecord.DependencyKey)
		if err != nil {
			return nil, err
		}
		if transactionRecord == nil {
			return nil, errors.Errorf("failed to find transaction by OperationID %s and DependencyKey %s", transactionRecord.OperationID, transactionRecord.DependencyKey)
		}

		txs.Transactions = append(txs.Transactions, transactionRecord)
	}

	return txs, nil
}
