package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/itransaction"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_account"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iaccount"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_offer"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ioffer"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iuser"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_user"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/icommission"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_comission"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iuserpreference"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_userpreference"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_market"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/imarket"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ikeyvalue"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_keyvalue"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/icode"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_code"
)

type TransactionManagerImpl struct {
	chaincode.IManager
	AccountCC                iaccount.IAccountManager
	CodeCC           		 icode.ICodeManager
	OfferCC                  ioffer.IOfferManager
	OfferStatusCC            ikeyvalue.IKeyValueManager
	BitcoinWithdrawalOfferCC ioffer.IOfferManager
	UserCC                   iuser.IUserManager
	CommissionCC             icommission.ICommissionManager
	UserPreferenceCC         iuserpreference.IUserPreferenceManager
	MarketCC                 imarket.IMarketManager
}

func GetTransactionManager(manager chaincode.IManager) itransaction.ITransactionManager {

	accountRemoteManager := remote_account.GetAccountRemoteManager(manager, invoke.AccountChannel, invoke.AccountCC)
	offerRemoteManager := remote_offer.GetOfferRemoteManager(manager, invoke.OfferChannel, invoke.OfferCC)
	offerStatusRemoteManager := remote_keyvalue.GetKeyValueRemoteManager(manager, invoke.OfferStatusChannel, invoke.OfferStatusCC)
	bitcoinWithdrawalOfferRemoteManager := remote_offer.GetOfferRemoteManager(manager, invoke.BitcoinWithdrawalOfferChannel, invoke.BitcoinWithdrawalOfferCC)
	userRemoteManager := remote_user.GetUserRemoteManager(manager, invoke.UserChannel, invoke.UserCC)
	commissionRemoteManager := remote_comission.GetCommissionRemoteManager(manager, invoke.CommissionChannel, invoke.CommissionCC)
	userPreferenceRemoteManager := remote_userpreference.GetUserPreferenceRemoteManager(manager, invoke.UserPreferenceChannel, invoke.UserPreferenceCC)
	marketRemoteManager := remote_market.GetMarketRemoteManager(manager, invoke.MarketChannel, invoke.MarketCC)
	codeRemoteManager := remote_code.GetCodeRemoteManager(manager, invoke.EthereumCodeChannel, invoke.EthereumCodeCC)

	// менеджеры для ввода средств по выписке
	// устанавливается в чейнкоде на ходу в зависимости от воходных параметров

	return &TransactionManagerImpl{
		manager,
		accountRemoteManager,
		codeRemoteManager,
		offerRemoteManager,
		offerStatusRemoteManager,
		bitcoinWithdrawalOfferRemoteManager,
		userRemoteManager,
		commissionRemoteManager,
		userPreferenceRemoteManager,
		marketRemoteManager,
	}
}
