package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/model/offerCC"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
)

func (this *TransactionManagerImpl) TransferFunds(transfer *transactionCC.Transfer) (*transactionCC.TransactionList, error) {
	operationID := transfer.OperationID

	transferUser, err := this.AccountCC.CheckAndLoadAccount(&transfer.UserAccountInfo)
	if err != nil {
		return nil, errors.Errorf("check and load account error %s", err)
	}

	if transferUser.Status != accountCC.AccountStatus_ACTIVE {
		return nil, errors.Errorf("check and load account is not active %s", transferUser)
	}

	transfer.UserAccountInfo = *transferUser.ToUserAccountInfo()

	transferTransactionInfoList, err := this.findTransactionListByOperationID(transfer)
	if err != nil {
		return nil, err
	}
	if transferTransactionInfoList == nil {
		return nil, errors.Errorf("Unable to find lock account by  operationID %s", operationID)
	}

	if len(transferTransactionInfoList.LockTransactionRecordInfoList) == 0 {
		return nil, errors.New("Size of transaction record is empty")
	}

	var processType transactionCC.ProcessType

	for _, lockTransactionRecordInfo := range transferTransactionInfoList.LockTransactionRecordInfoList {
		// проверка что все транзакции из списка имеют один ProcessType и operationID
		// при условии что для типа процесса "Транзакция комиссия с получателя" комиссия не указана
		err = lockTransactionRecordInfo.Validate()
		if err != nil {
			return nil, err
		}

		if len(processType) != 0 && processType != lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType {
			return nil, chaincode.GlobalLogger.Errorf("transaction can't have different ProcessType. first %s second %s",
				processType, lockTransactionRecordInfo.InLockCommissionTransactionRecord.ProcessType)
		}

		processType = lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType
	}

	if processType == transactionCC.ExchangeWithBothCommission_ProcessType {
		keysWithFilter := &requests.KeysWithFilter{
			Keys:      []string{transfer.OperationID, offerCC.OfferAccepted.String()},
			FilterKeys: []string{},
		}
		operationIDStatusKeyValue, err := this.OfferStatusCC.GetKeyValueArrayByKeysWithFilter(keysWithFilter)
		if err != nil {
			return nil, errors.Errorf("There is no approved offer for give operation id %s. Error %s", operationID, err.Error())
		}
		if operationIDStatusKeyValue == nil ||
			len(operationIDStatusKeyValue.KeyValue) != 1 ||
			operationIDStatusKeyValue.KeyValue[0].Value != offerCC.OfferAccepted.String() {
			return nil, errors.Errorf("There is no approved offer for give operation id %s", operationID)
		}
	}

	userTransactionsList := &transactionCC.TransactionList{}

	for _, lockTransactionRecordInfo := range transferTransactionInfoList.LockTransactionRecordInfoList {

		transactionRecordList, err := this.transferFunds(&lockTransactionRecordInfo)
		if err != nil {
			return userTransactionsList, err
		}

		userTransactionsList.Transactions = append(userTransactionsList.Transactions, transactionRecordList.Transactions...)
	}

	return userTransactionsList, nil
}
