package mngr

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"

	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
)

func (this *TransactionManagerImpl) UnlockFundsOnAccount(transfer *transactionCC.Transfer) (*transactionCC.TransactionList, error) {
	operationID := transfer.OperationID

	transferUser, err := this.AccountCC.CheckAndLoadAccount(&transfer.UserAccountInfo)
	if err != nil {
		return nil, errors.Errorf("check and load account error %s", err)
	}

	if transferUser.Status != accountCC.AccountStatus_ACTIVE {
		return nil, errors.Errorf("check and load account is not active %s", transferUser)
	}

	transfer.UserAccountInfo = *transferUser.ToUserAccountInfo()

	unlockTransactionInfoList, err := this.findTransactionListByOperationID(transfer)
	if err != nil {
		return nil, err
	}
	if unlockTransactionInfoList == nil {
		return nil, errors.Errorf("Unable to find lock account by  operationID %s", operationID)
	}

	if len(unlockTransactionInfoList.LockTransactionRecordInfoList) == 0 {
		return nil, errors.Errorf("Size of transaction record is empty")
	}

	var processType transactionCC.ProcessType

	chaincode.GlobalLogger.Infof("UnlockTransactionInfo size: ", len(unlockTransactionInfoList.LockTransactionRecordInfoList))

	for _, lockTransactionRecordInfo := range unlockTransactionInfoList.LockTransactionRecordInfoList {
		chaincode.GlobalLogger.Infof("check operation id for transactionRecord with key %s", lockTransactionRecordInfo.InLockAmountTransactionRecord.Key)
		if lockTransactionRecordInfo.InLockAmountTransactionRecord.OperationID != operationID {
			chaincode.GlobalLogger.Error("Cannot process funds transfer for different operation types")
			return nil, errors.Errorf("Cannot process funds transfer for different operation types")
		}

		chaincode.GlobalLogger.Infof("check operation id for transactionRecord with key %s", lockTransactionRecordInfo.InLockCommissionTransactionRecord.Key)
		if lockTransactionRecordInfo.InLockCommissionTransactionRecord.OperationID != operationID {
			chaincode.GlobalLogger.Error("Cannot process funds transfer for different operation types")
			return nil, errors.Errorf("Cannot process funds transfer for different operation types")
		}

		chaincode.GlobalLogger.Infof("check operation id for transactionRecord with key %s", lockTransactionRecordInfo.OutLockAmountTransactionRecord.Key)
		if lockTransactionRecordInfo.OutLockAmountTransactionRecord.OperationID != operationID {
			chaincode.GlobalLogger.Error("Cannot process funds transfer for different operation types")
			return nil, errors.Errorf("Cannot process funds transfer for different operation types")
		}

		// проверка ProcessType
		if lockTransactionRecordInfo.InLockAmountTransactionRecord != nil && lockTransactionRecordInfo.OutLockAmountTransactionRecord != nil &&
			lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType != lockTransactionRecordInfo.OutLockAmountTransactionRecord.ProcessType {
			return nil, errors.Errorf("ProcessType not equals InLockAmountTransactionRecord %s and OutLockAmountTransactionRecord %s",
				lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType,
				lockTransactionRecordInfo.OutLockAmountTransactionRecord.ProcessType)
		}

		if lockTransactionRecordInfo.InLockCommissionTransactionRecord != nil && lockTransactionRecordInfo.OutLockCommissionTransactionRecord != nil &&
			lockTransactionRecordInfo.InLockCommissionTransactionRecord.ProcessType != lockTransactionRecordInfo.OutLockCommissionTransactionRecord.ProcessType {
			return nil, errors.Errorf("ProcessType not equals InLockCommissionTransactionRecord %s and OutLockCommissionTransactionRecord %s",
				lockTransactionRecordInfo.InLockCommissionTransactionRecord.ProcessType,
				lockTransactionRecordInfo.OutLockCommissionTransactionRecord.ProcessType)
		}

		if lockTransactionRecordInfo.InLockCommissionTransactionRecord != nil && lockTransactionRecordInfo.InLockAmountTransactionRecord != nil &&
			lockTransactionRecordInfo.InLockCommissionTransactionRecord.ProcessType != lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType {
			return nil, errors.Errorf("ProcessType not equals InLockCommissionTransactionRecord %s and InLockAmountTransactionRecord %s",
				lockTransactionRecordInfo.InLockCommissionTransactionRecord.ProcessType,
				lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType)
		}

		if lockTransactionRecordInfo.OutLockCommissionTransactionRecord != nil && lockTransactionRecordInfo.OutLockAmountTransactionRecord != nil &&
			lockTransactionRecordInfo.OutLockCommissionTransactionRecord.ProcessType != lockTransactionRecordInfo.OutLockAmountTransactionRecord.ProcessType {
			return nil, errors.Errorf("ProcessType not equals OutLockCommissionTransactionRecord %s and OutLockAmountTransactionRecord %s",
				lockTransactionRecordInfo.OutLockCommissionTransactionRecord.ProcessType,
				lockTransactionRecordInfo.OutLockAmountTransactionRecord.ProcessType)
		}

		processType = lockTransactionRecordInfo.InLockAmountTransactionRecord.ProcessType
	}

	if processType == transactionCC.ExchangeWithBothCommission_ProcessType {
		//offerRequest, err := this.OfferCC.GetOffer(&requests.Id{Id: operationID})
		//if err != nil {
		//	return nil, errors.Errorf("Error GetOffer by operationId %s. Error: %s", operationID, err.Error())
		//}
		//if offerRequest.OfferRecordStatus != offerCC.OfferRejected &&
		//	offerRequest.OfferRecordStatus != offerCC.OfferCanceled {
		//	return nil, errors.Errorf("Error OfferRequest by operationId %s status: %s != OfferRejected or OfferCanceled", offerRequest.OfferRequestStatus)
		//}
	}

	transferFundsResponseList := &transactionCC.TransactionList{}
	for _, lockTransactionRecordInfo := range unlockTransactionInfoList.LockTransactionRecordInfoList {
		transactionRecordList, err := this.unlockTransferFunds(&lockTransactionRecordInfo)
		if err != nil {
			return transferFundsResponseList, err
		}

		transferFundsResponseList.Transactions = append(transferFundsResponseList.Transactions, transactionRecordList.Transactions...)
	}

	return transferFundsResponseList, nil
}
