package marketCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	model "gitlab.smartblocklab.com/smartcontracts/common/model/accountCC"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
)

type DocumentType string

func (documentType DocumentType) String() string {
	return string(documentType)
}

type SessionID int64

func (sessionID SessionID) String() string {
	return string(sessionID)
}

type OrderStatus string

func (orderStatus OrderStatus) String() string {
	return string(orderStatus)
}

type OrderType int

func (orderType OrderType) String() string {
	return string(orderType)
}

type MarketFundsArg struct {
	AccountID   string                   `json:"account_id"`
	OperationID string                   `json:"operation_id"`
	UserID      string                   `json:"user_id"`
	Amount      nums.PlainCurrencyNumber `json:"amount"`
}

type FindMarketTransactionArg struct {
	OperationID   string                      `json:"operation_id"`
	UserID        string                      `json:"user_id"`
	CurrencyType  string                      `json:"currency_type"`
	AccountID     string                      `json:"account_id"`
	OperationType transactionCC.OperationType `json:"operation_type"`
}

type FindTransactionsByOperationIDArg struct {
	OperationID string `json:"operation_id"`
}

func (record MarketTransaction) String() string {
	result := "\n\n| SessionID: " + record.SessionID.String() + " | " +
		"| DocumentType: " + record.DocumentType.String() + " | " +
		//"| Amount: " + record.Amount.String() + " | " +
		"| InOutType: " + record.InOutType.String() + " | " +
		"| OperationType: " + record.OperationType.String() + " | " +
		"| ProcessType: " + record.ProcessType.String() + " | " +
		"| UserAccountInfo: " + record.UserAccountInfo.String() + " | "

	if record.DestinationUserAccountInfo != nil {
		result += "| DestinationUserAccountInfo: " + record.DestinationUserAccountInfo.String() + " | "
	}

	result += "| Timestamp: " + record.Timestamp.String() + " | " +
		"| DependencyKey: " + record.DependencyKey + " | " +
		"| Key: " + record.Key + " | " +
		"| HashCode: " + record.HashCode + " | "

	return result
}

type MarketTransactionList struct {
	MarketTransactions []MarketTransaction `json:"market_transactions"`
}

func (list *MarketTransactionList) Print() {
	for _, marketTransaction := range list.MarketTransactions {
		logging.Get().Info(marketTransaction.String())
	}
}

type MarketArg struct {
	UserID       string `json:"user_id"`
	CurrencyType string `json:"currency_type"`
	AccountID    string `json:"account_id"`
}

type UIMarketOrder struct {
	AccountID  string                   `json:"account_id"`
	Price      nums.PlainCurrencyNumber `json:"price"`
	Amount     nums.PlainCurrencyNumber `json:"amount"`
	StopPrice  nums.PlainCurrencyNumber `json:"stop_price"`
	LimitPrice nums.PlainCurrencyNumber `json:"limit_price"`
}

type UIDepositFunds struct {
	MarketAccountID string                   `json:"market_account_id"`
	AccountID       string                   `json:"account_id"`
	Amount          nums.PlainCurrencyNumber `json:"amount"`
}

type UIMarketWithdrawal struct {
	Market      *model.TxnAccountInfo    `json:"market"`      // от куда списываем деньги с биржи
	Transaction *model.TxnAccountInfo    `json:"transaction"` // на какой счёт переводим средства
	Amount      nums.PlainCurrencyNumber `json:"amount"`      // сколько?
}

type PlainMarketOrderList struct {
	PlainMarketOrders []PlainMarketOrder `json:"plain_market_orders"`
}

type PlainMarketOrder struct {
	UserID       string                   `json:"user_id"`
	OperationID  string                   `json:"operation_id"`
	SessionID    SessionID                `json:"session_id"`
	AccountID    string                   `json:"account_id"`
	SellPrice    nums.PlainCurrencyNumber `json:"sell_price"`
	DocumentType DocumentType             `json:"document_type"`
	OrderStatus  OrderStatus              `json:"order_status"`
	BuyAmount    nums.PlainCurrencyNumber `json:"buy_amount"`
	FilledAmount nums.PlainCurrencyNumber `json:"filled_amount"`
	StopPrice    nums.PlainCurrencyNumber `json:"stop_price"`
	LimitPrice   nums.PlainCurrencyNumber `json:"limit_price"`
	Timestamp    transactionCC.Timestamp  `json:"timestamp"`
}

type MarketBalance struct {
	UserID    string                   `json:"user_id"`
	Balance   nums.PlainCurrencyNumber `json:"balance"`
	AccountID string                   `json:"account_id"`
}

type TransactionList struct {
	OrderTransactions  []OrderTransaction  `json:"order_transactions"`
	MarketTransactions []MarketTransaction `json:"market_transactions"`
}

/**
результат матчинга ордеров
*/
type OrderTransaction struct {
	Key           string                   `json:"key"`
	SequenceID    int                      `json:"sequence_id"`
	SessionID     SessionID                `json:"session_id"`
	OutOrderID    string                   `json:"out_order_id"`
	InOrderID     string                   `json:"in_order_id"`
	Price         nums.PlainCurrencyNumber `json:"price"`
	Amount        nums.PlainCurrencyNumber `json:"amount"`
	DocumentType  DocumentType             `json:"document_type"`
	DependencyKey string                   `json:"dependency_key"`
}

type MarketOrder struct {
	SessionID      SessionID
	UserID         string
	OperationID    string
	StatusInternal OrderStatus // статус ордера: открыт/закрыт
	AccountID      string
	CurrencyPair   *CurrencyPair           // валютная пара
	OrderType      OrderType               // тип лимитного ордера: http://profitraders.com/Bases/Orders.html
	Price          *nums.CurrencyNumber    // цена 1 единицы валюты, по какой цене хочу купить
	Amount         *nums.CurrencyNumber    // количество единиц валюты, сколько хочу купить
	FilledAmount   *nums.CurrencyNumber    // сколько валюты было обработано этим ордером
	Timestamp      transactionCC.Timestamp // дата создания ордера

	// Граница при продаже:
	// если текущая цена стала ниже StopPrice, то сразу же продаем
	StopPrice *nums.CurrencyNumber

	// Граница при продаже:
	// если текущая цена стала выше LimitPrice, то сразу же продаем
	LimitPrice *nums.CurrencyNumber
}

type MarketOrderList struct {
	MarketOrders []MarketOrder `json:"market_orders"`
}

// Описатель котировки
type CurrencyPair struct {
	BuyAsset  *nums.Asset `json:"buy_asset"`  // Что хотим купить
	SellAsset *nums.Asset `json:"sell_asset"` // Что хотим продать
}

// Опции лимитированного ордера
type Options struct {
	// Граница при продаже:
	// если текущая цена стала ниже StopPrice, то сразу же продаем
	StopPrice *nums.CurrencyNumber

	// Граница при продаже:
	// если текущая цена стала выше LimitPrice, то сразу же продаем
	LimitPrice *nums.CurrencyNumber
}
