package userPreferenceCC

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_user"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"encoding/json"
	"reflect"
)

type UserPreference struct {
	preferenceCC.Preference
}

type UserPreferenceArray struct {
	UserPreference []UserPreference `json:"user_preference_array,omitempty"`
}

func UnmarshalUserPreferenceArray(data []byte) (*UserPreferenceArray, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj UserPreferenceArray
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func (this UserPreference) CheckBeforeAdd(manager chaincode.IManager) error {
	userRemoteManager := remote_user.GetUserRemoteManager(manager, invoke.UserChannel, invoke.UserCC)
	keysUserId := this.Keys[0]
	user, err := userRemoteManager.GetUser(&requests.Id{Id: keysUserId})
	if err != nil {
		return err
	}
	if user.UserID != keysUserId {
		return errors.Errorf("found userID %s not eq with expected userID %s", user.UserID, keysUserId)
	}

	return nil
}

func (this UserPreference) GetObjectType() string  {
	return "UserPreference";
}

func (this UserPreference) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	return this.CheckBeforeAdd(manager)
}

func (this UserPreference) New() chaincode.ValidObject {
	return &UserPreference{}
}

func Unmarshal(data []byte) (*UserPreference, error) {
	if data == nil {
		return nil, chaincode.GlobalLogger.Error("can't Unmarshal nil data")
	}

	var obj UserPreference
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *UserPreference) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*UserPreference, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check validObjectc is not reference")
	}

	casted, ok := validObject.(*UserPreference)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
