package nums

import "strconv"

func (this *CurrencyNumber) DivRaw(b *CurrencyNumber) (*CurrencyNumber, error) {

	bNumber, err := NumFromString(b.RoundBankString(), this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	return this.Div(bNumber)
}

func (this *CurrencyNumber) DivInt(b uint64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromInt(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	return this.Div(currencyNumber)
}

func (this *CurrencyNumber) DivFloat(b float64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromFloat(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	return this.Div(currencyNumber)
}

func (this *CurrencyNumber) Percent(asset *Asset, percent uint) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromString(strconv.Itoa(int(percent)), asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	base, err := NumFromString(strconv.Itoa(100), asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	resultMul, err := this.Mul(currencyNumber)
	if err != nil {
		return nil, err
	}

	return resultMul.Div(base)
}

func (this *CurrencyNumber) Equals(b *CurrencyNumber) (bool, error) {
	err := this.AssetEquals(b)
	if err != nil {
		return false, err
	}
	return this.Num.Cmp(b.Num) == 0, nil
}

func (this *CurrencyNumber) GreaterEq(b *CurrencyNumber) (bool, error) {
	_, err := this.Validate()
	if err != nil {
		return false, err
	}

	_, err = b.Validate()
	if err != nil {
		return false, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return false, err
	}

	return this.Num.Cmp(b.Num) >= 0, nil
}

func (this *CurrencyNumber) Greater(b *CurrencyNumber) (bool, error) {
	_, err := this.Validate()
	if err != nil {
		return false, err
	}

	_, err = b.Validate()
	if err != nil {
		return false, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return false, err
	}

	return this.Num.Cmp(b.Num) > 0, nil
}

func (this *CurrencyNumber) LessEq(b *CurrencyNumber) (bool, error) {
	_, err := this.Validate()
	if err != nil {
		return false, err
	}

	_, err = b.Validate()
	if err != nil {
		return false, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return false, err
	}

	return this.Num.Cmp(b.Num) <= 0, nil
}

func (this *CurrencyNumber) Less(b *CurrencyNumber) (bool, error) {
	_, err := this.Validate()
	if err != nil {
		return false, err
	}

	_, err = b.Validate()
	if err != nil {
		return false, err
	}

	err = this.AssetEquals(b)
	if err != nil {
		return false, err
	}

	return this.Num.Cmp(b.Num) < 0, nil
}

func (this *CurrencyNumber) Inc() (*CurrencyNumber, error) {
	one, err := One(this)
	if err != nil {
		return nil, err
	}
	return this.AddSelf(one)
}

func (this *CurrencyNumber) Dec() (*CurrencyNumber, error) {
	one, err := One(this)
	if err != nil {
		return nil, err
	}
	return this.SubSelf(one)
}

func (this *CurrencyNumber) MulRaw(b *CurrencyNumber) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromString(b.RoundBankString(), this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	return this.Mul(currencyNumber)
}

func (this *CurrencyNumber) MulInt(b uint64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromInt(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	return this.Mul(currencyNumber)
}

func (this *CurrencyNumber) MulFloat(b float64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromFloat(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	return this.Mul(currencyNumber)
}

func (this *CurrencyNumber) MulSelfFloat(b float64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromFloat(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	return this.MulSelf(currencyNumber)
}

func (this *CurrencyNumber) MulSelfInt(b uint64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromInt(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	return this.MulSelf(currencyNumber)
}

func (this *CurrencyNumber) DivSelfFloat(b float64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromFloat(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}

	resutl, err := this.DivSelf(currencyNumber)
	if err != nil {
		return nil, err
	}
	return resutl, nil
}


func (this *CurrencyNumber) DivSelfInt(b uint64) (*CurrencyNumber, error) {
	currencyNumber, err := NumFromInt(b, this.Asset.CurrencyType)
	if err != nil {
		return nil, err
	}
	result, err := this.DivSelf(currencyNumber)
	if err != nil {
		return nil, err
	}
	return result, nil
}
