package remote_code

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/keyValueCC"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
	"gitlab.smartblocklab.com/smartcontracts/common/model/codeCC"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/icode"
)

type ICodeRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetCodeRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) icode.ICodeManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	return &ICodeRemoteManager{remoteManagerImpl}
}

func (this ICodeRemoteManager) AddCode(userId string, currencyNumber *nums.PlainCurrencyNumber, codeInfo *codeCC.CodeInfo) (*keyValueCC.KeyValue, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ICodeRemoteManager) AddKeyValue(keyValue *keyValueCC.KeyValue) (*keyValueCC.KeyValue, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ICodeRemoteManager) AddKeyValueArray(keyValueArray *keyValueCC.KeyValueArray) (*keyValueCC.KeyValueArray, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ICodeRemoteManager) GetKeyValue(keyId *requests.Id) (*keyValueCC.KeyValue, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueFunction, keyId); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.Unmarshal(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}

func (this ICodeRemoteManager) GetKeyValueArray(keyArrayId *requests.ArrayId) (*keyValueCC.KeyValueArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueArrayFunction, keyArrayId); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.UnmarshalKeyValueArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}

func (this ICodeRemoteManager) GetKeyValueArrayByKeysWithFilter(keysWithFilter *requests.KeysWithFilter) (*keyValueCC.KeyValueArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueArrayByKeysWithFilterFunction, keysWithFilter); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.UnmarshalKeyValueArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}
