package remote_comission

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model/commissionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/icommission"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

type CommissionRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetCommissionRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) icommission.ICommissionManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	ComissionRemoteManager := &CommissionRemoteManager{remoteManagerImpl}
	return ComissionRemoteManager
}

func (this CommissionRemoteManager) AddCommission(commission *commissionCC.Commission, systemParameterUserID string) (*commissionCC.Commission, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this CommissionRemoteManager) UpdateCommission(commission *commissionCC.Commission, systemParameterUserID string) (*commissionCC.Commission, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this CommissionRemoteManager) GetCommission(getCommissionArg commissionCC.GetCommissionArg) (*commissionCC.Commission, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetCommissionFunction, getCommissionArg); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result commissionCC.Commission
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
