package remote_cryptocurrency

import (
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/icryptocurrency"
	"gitlab.smartblocklab.com/smartcontracts/common/model/cryptocurrencyCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/transactionCC"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
)

type ICryptocurrencyRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetCryptocurrencyRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) icryptocurrency.ICryptocurrencyManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	cryptocurrencyRemoteManager := &ICryptocurrencyRemoteManager{remoteManagerImpl}
	return cryptocurrencyRemoteManager
}

func (this ICryptocurrencyRemoteManager) AddCryptocurrencies(daemonName string, cryptocurrencyListInput cryptocurrencyCC.CryptocurrencyListInput) (interface{}, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ICryptocurrencyRemoteManager) ClearTransferredCryptocurrencyList(daemonName *requests.Id) (*transactionCC.CheckExistsResult, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ICryptocurrencyRemoteManager) GetCryptocurrenciesByDaemonName(daemonName *requests.Id) (*cryptocurrencyCC.CryptocurrencyStoreOutput, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetCryptocurrenciesByDaemonNameFunction, daemonName); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result cryptocurrencyCC.CryptocurrencyStoreOutput
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
