package remote_keyvalue

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/keyValueCC"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ikeyvalue"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

type IKeyValueRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetKeyValueRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) ikeyvalue.IKeyValueManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	return &IKeyValueRemoteManager{remoteManagerImpl}
}

func (this IKeyValueRemoteManager) AddKeyValue(keyValue *keyValueCC.KeyValue) (*keyValueCC.KeyValue, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this IKeyValueRemoteManager) AddKeyValueArray(keyValueArray *keyValueCC.KeyValueArray) (*keyValueCC.KeyValueArray, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this IKeyValueRemoteManager) GetKeyValue(keyId *requests.Id) (*keyValueCC.KeyValue, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueFunction, keyId); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.Unmarshal(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}

func (this IKeyValueRemoteManager) GetKeyValueArray(keyArrayId *requests.ArrayId) (*keyValueCC.KeyValueArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueArrayFunction, keyArrayId); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.UnmarshalKeyValueArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}

func (this IKeyValueRemoteManager) GetKeyValueArrayByKeysWithFilter(keysWithFilter *requests.KeysWithFilter) (*keyValueCC.KeyValueArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetKeyValueArrayByKeysWithFilterFunction, keysWithFilter); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := keyValueCC.UnmarshalKeyValueArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}
