package remote_market

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/imarket"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"gitlab.smartblocklab.com/smartcontracts/common/model/marketCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

type MarketRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetMarketRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) imarket.IMarketManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	marketRemoteManager := &MarketRemoteManager{remoteManagerImpl}
	return marketRemoteManager
}

func (this MarketRemoteManager) DepositFunds(userID string, inputMarketFoundsObject marketCC.UIDepositFunds, operationID requests.Id) (*marketCC.MarketTransaction, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this MarketRemoteManager) GetBalance(balanceArg marketCC.MarketArg) (*marketCC.MarketBalance, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.GetBalanceFunction, balanceArg)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.MarketBalance{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}

func (this MarketRemoteManager) LockWithdrawalFunds(userID string, arg marketCC.UIMarketWithdrawal, operationID requests.Id) (*marketCC.MarketTransactionList, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}

func (this MarketRemoteManager) TransferWithdrawalFunds(userID string, operationID requests.Id) (*marketCC.MarketTransactionList, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this MarketRemoteManager) CreateOrder(userID string, marketOrder marketCC.UIMarketOrder, operationID requests.Id) (*marketCC.MarketTransactionList, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this MarketRemoteManager) GetAllOrders() (*marketCC.PlainMarketOrderList, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.GetAllOrdersFunction, nil)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.PlainMarketOrderList{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}
func (this MarketRemoteManager) GetUserOrders(arg requests.Id) (*marketCC.PlainMarketOrderList, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.GetUserOrdersFunction, arg)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.PlainMarketOrderList{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}
func (this MarketRemoteManager) CancelOrder(UserID string, id requests.Id) (*marketCC.MarketTransactionList, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this MarketRemoteManager) MatchOrders() (*marketCC.TransactionList, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this MarketRemoteManager) FindOutgoingTransactions(arg marketCC.FindMarketTransactionArg) (*marketCC.MarketTransactionList, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.FindOutgoingTransactionsFunction, arg)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.MarketTransactionList{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}

func (this MarketRemoteManager) FindIncomingTransactions(arg marketCC.FindMarketTransactionArg) (*marketCC.MarketTransactionList, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.FindIncomingTransactionsFunction, arg)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.MarketTransactionList{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}

func (this MarketRemoteManager) FindTransactionsByOperationID(arg marketCC.FindTransactionsByOperationIDArg) (*marketCC.MarketTransactionList, error) {
	resp := this.SignedInvokeChaincode(this.GetChannelName(), invoke.MarketaccountCC, invoke.FindTransactionsByOperationIDFunction, arg)

	if resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	}

	result := marketCC.MarketTransactionList{}

	if err := json.Unmarshal(resp.Payload, &result); err != nil {
		return nil, logging.Get().Errorf("Failed to unmarshal result, due to %s", err.Error())
	}

	return &result, nil
}
