package remote_offer

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/ioffer"
	"gitlab.smartblocklab.com/smartcontracts/common/model/offerCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
)

type OfferRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetOfferRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) ioffer.IOfferManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	return &OfferRemoteManager{remoteManagerImpl}
}

// invoke
func (this OfferRemoteManager) AddOffer(outOfferRequest *offerCC.OfferRecord, inOfferRequest *offerCC.OfferRecord, operationID *requests.Id) (*offerCC.OfferRecords, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this OfferRemoteManager) GetOffers(operationID *requests.Id) (*offerCC.Offers, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetOffersFunction, operationID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result offerCC.Offers
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this OfferRemoteManager) GetOfferRecords(operationID *requests.Id) (*offerCC.OfferRecords, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetOfferRecordsFunction, operationID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result offerCC.OfferRecords
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
