package remote_systempreference

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/isystempreference"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
)

type ISystempreferenceRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetSystempreferenceRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) isystempreference.ISystemPreferenceManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{iManager, channelName, chaincodeName}
	return &ISystempreferenceRemoteManager{remoteManagerImpl}
}

func (this ISystempreferenceRemoteManager) AddPreference(preference *preferenceCC.Preference) (*preferenceCC.Preference, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ISystempreferenceRemoteManager) UpdatePreference(preference *preferenceCC.Preference) (*preferenceCC.Preference, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this ISystempreferenceRemoteManager) GetPreference(key *requests.Id) (*preferenceCC.Preference, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetPreferenceFunction, key); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result preferenceCC.Preference
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this ISystempreferenceRemoteManager) GetPreferenceArrayByKeysWithFilter(keysWithFilter *requests.KeysWithFilter) (*preferenceCC.PreferenceArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetPreferenceArrayByKeysWithFilterFunction, keysWithFilter); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := preferenceCC.UnmarshalPreferenceArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}
