package remote_user

import (
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iuser"
	"gitlab.smartblocklab.com/smartcontracts/common/model/userCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
)

type UserRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetUserRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) iuser.IUserManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	userRemoteManager := &UserRemoteManager{remoteManagerImpl}
	return userRemoteManager
}

func (this UserRemoteManager) AddRootUser(user *userCC.User) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) AddOrganizationRootUser(user *userCC.User) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) AddSystemUser(user *userCC.User) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) AddServiceUser(user *userCC.User) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) AddUser(user *userCC.User) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) AddPublicKey(addPublicKeyArg userCC.AddPublicKeyArg) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) RevokeUserPublicKey(revokeUserPublicKeyArg userCC.RevokeUserPublicKeyArg) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) ActivateUser(userID *requests.Id) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}
func (this UserRemoteManager) BlockUser(userID *requests.Id) (chaincode.ValidObject, error) {
	return nil, errors.New("Failed to make remote call")
}

// query
func (this UserRemoteManager) GetUser(userID *requests.Id) (*userCC.User, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetUserFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result userCC.User
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}
func (this UserRemoteManager) VerifyUserSignature(verifyUserSignatureArg requests.VerifyUserSignatureArg) (chaincode.ValidObject, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.VerifyUserSignatureFunction, verifyUserSignatureArg); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		return nil, nil
	}
}
