package remote_userpreference

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/userPreferenceCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/iuserpreference"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

type IUserPreferenceRemoteManager struct {
	*chaincode.RemoteManagerImpl
}

func GetUserPreferenceRemoteManager(iManager chaincode.IManager, channelName string, chaincodeName string) iuserpreference.IUserPreferenceManager {
	remoteManagerImpl := &chaincode.RemoteManagerImpl{IManager: iManager, ChannelName: channelName, ChaincodeName: chaincodeName}
	userPreferenceRemoteManager := &IUserPreferenceRemoteManager{remoteManagerImpl}
	return userPreferenceRemoteManager
}

func (this IUserPreferenceRemoteManager) AddPreference(preference *userPreferenceCC.UserPreference) (*userPreferenceCC.UserPreference, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this IUserPreferenceRemoteManager) UpdatePreference(preference *userPreferenceCC.UserPreference) (*userPreferenceCC.UserPreference, error) {
	return nil, errors.New("Failed to make remote call")
}

func (this IUserPreferenceRemoteManager) GetPreference(userID *requests.Id) (*userPreferenceCC.UserPreference, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetPreferenceFunction, userID); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		var result userPreferenceCC.UserPreference
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, errors.New(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this IUserPreferenceRemoteManager) GetUserPreferenceArrayByKeysWithFilter(keysWithFilter *requests.KeysWithFilter) (*userPreferenceCC.UserPreferenceArray, error) {
	if resp := this.SignedInvokeChaincode(this.GetChannelName(), this.GetChaincodeName(), invoke.GetUserPreferenceArrayByKeysWithFilterFunction, keysWithFilter); resp.Status != 200 {
		return nil, errors.New(resp.Message)
	} else {
		result, err := userPreferenceCC.UnmarshalUserPreferenceArray(resp.Payload)
		if err != nil {
			return nil, errors.New(err.Error())
		} else {
			return result, nil
		}
	}
}